package org.apidesign.apifest08.test;

import junit.framework.*;
import org.apidesign.apifest08.currency.*;

import java.util.*;

/** Finish the Convertor API, and then write bodies of methods inside
 * of this class to match the given tasks. To fullfil your task, use the
 * API define in the <code>org.apidesign.apifest08.currency</code> package.
 * Do not you reflection, or other hacks as your code
 * shall run without any runtime permissions.
 */
public class Task1Test extends TestCase {

	private static final String
			USD = "USD",
			CZK = "CZK",
			SKK = "SKK";

	private static final Currency
			USD2 = Currency.getInstance(USD),
			CZK2 = Currency.getInstance(CZK),
			SKK2 = Currency.getInstance(SKK);

	public Task1Test(String testName) {
    	super(testName);
    }

    @Override
    protected void setUp() throws Exception {
	}

    @Override
    protected void tearDown() throws Exception {
    }

    /** Create convertor that understands two currencies, CZK and
     *  USD. Make 1 USD == 17 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     *
     * @return prepared convertor ready for converting USD to CZK and CZK to USD
     */
    public static CurrencyConverter createCZKtoUSD() {
		return OfflineConverterProvider.getInstance().getConverter(17, CZK, 1, USD);
    }

	public static CurrencyConverter createCZKtoUSD2() {
		return OfflineConverterProvider.getInstance().getConverter(CZK, USD);
    }

    /** Create convertor that understands two currencies, CZK and
     *  SKK. Make 100 SKK == 80 CZK.
     *
     * Creation of the convertor shall not require subclassing of any class
     * or interface on the client side.
     * 
     * @return prepared convertor ready for converting SKK to CZK and CZK to SKK
     */
    public static CurrencyConverter createSKKtoCZK() {
        return OfflineConverterProvider.getInstance().getConverter(100, SKK, 80, CZK);
    }

	public static CurrencyConverter createSKKtoCZK2() {
        return OfflineConverterProvider.getInstance().getConverter(SKK, CZK);
    }
    
    /** Use the convertor from <code>createCZKtoUSD</code> method and do few conversions
     * with it.
     */
    public void testCurrencyCZKUSD() throws Exception {
		CurrencyConverter c = createCZKtoUSD();
		testCZKUSD(c);

		// test without specifying rates
		c = createCZKtoUSD2();
		testCZKUSD(c);
	}

	private void testCZKUSD(CurrencyConverter c) {
		double czk, usd;
		// convert $5 to CZK using c:
		// assertEquals("Result is 85 CZK");
		czk = c.convert(5, USD, CZK);
		assertEquals(85.0, czk, 0.0);
		czk = c.convert(5, USD2, CZK2);
		assertEquals(85.0, czk, 0.0);

		// convert $8 to CZK
		// assertEquals("Result is 136 CZK");
		czk = c.convert(8, USD, CZK);
		assertEquals(136.0, czk, 0.0);
		czk = c.convert(8, USD2, CZK2);
		assertEquals(136.0, czk, 0.0);

		// convert 1003CZK to USD
		// assertEquals("Result is 59 USD");
		usd = c.convert(1003, CZK, USD);
		assertEquals(59.0, usd, 0.0);
		usd = c.convert(1003, CZK2, USD2);
		assertEquals(59.0, usd, 0.0);
	}

	/** Use the convertor from <code>createSKKtoCZK</code> method and do few conversions
     * with it.
     */
    public void testCurrencySKKCZK() throws Exception {
		CurrencyConverter c = createSKKtoCZK();
		testCZKSKK(c);

		// test without specifying rates
		c = createSKKtoCZK2();
		testCZKSKK(c);
	}

	private void testCZKSKK(CurrencyConverter c) {
		double czk, skk;
		// convert 16CZK using c:
		// assertEquals("Result is 20 SKK");
		skk = c.convert(16, CZK, SKK);
		assertEquals(20.0, skk, 0.0);
		skk = c.convert(16, CZK2, SKK2);
		assertEquals(20.0, skk, 0.0);

		// convert 500SKK to CZK
		// assertEquals("Result is 400 CZK");
		czk = c.convert(500, SKK, CZK);
		assertEquals(400.0, czk, 0.0);
		czk = c.convert(500, SKK2, CZK2);
		assertEquals(400.0, czk, 0.0);
	}
}